class ChromatographException(Exception):
    """
    Common cromatograph exceptions
    """

class ChromatographStateException(ChromatographException):
    """
    Exceptions related to chromatograph state
    """
