class FurnaceProtocolException(Exception):
    """
    Exceptions related to connection protocol.
    """

class FurnaceConnectionException(Exception):
    """
    Exceptions related to connection issues.
    """

class FurnaceStateException(Exception):
    """
    Exceptions related to wrong furnace state.
    """
