class Activity():
    """
    Wrapper for activity data storage. Not implemented at the moment.
    """

    def __init__(self):
        """
        Initialize activity object. Not implemented at the moment
        """
        raise NotImplementedError()
